/*****************************\
*                             *
*  Implements TCADFramedText  *
*                             *
\*****************************/

#include <stdlib.h>
#include "framedtext.h"
#include "view.h"
#include "sheet.h"

#define TOP_OFFSET 3
#define BOTTOM_OFFSET 3
#define LEFT_OFFSET 3
#define RIGHT_OFFSET 3

TCADFramedText::TCADFramedText(TCADSheet *Sheet) : TCADText(Sheet)
{
	DTP("TCADFramedText::TCADFramedText()\n");

}

void
TCADFramedText::Save(TCADSaveStream *Fl)
{
	DTP("TCADFramedText::Save()\n");

	if ((Fl->SaveOnlySelected) && !(Selected)) return;

  Fl->SaveObjectName("TCADFramedText");
  SaveProperties(Fl);
  Fl->SaveEndObject();
}

void
TCADFramedText::SaveProperties(TCADSaveStream *Fl)
{
	DTP("TCADFramedText::SaveProperties()\n");

  TCADLinePropertiesI::SaveLineProperties(Fl);
  TCADText::SaveProperties(Fl);
}

char
TCADFramedText::LoadProperty(TCADLoadStream *Fl,char *ID)
{
	DTP("TCADFramedText::LoadProperty()\n");

  if (TCADLinePropertiesI::LoadProperty(Fl,ID)) return(TRUE);
  else return(TCADText::LoadProperty(Fl,ID));
}

void
TCADFramedText::Draw(TCADView *View,GdkRegion *Region,int MagA,int MagB)
{
//	DTP("TCADFramedText::Draw()\n");

  if (Region)
   if (gdk_region_rect_in(Region, &EncapRect) == GDK_OVERLAP_RECTANGLE_OUT) return;

  TCADText::Draw(View,Region,MagA,MagB);

  // Draw rectangle
	SetupGC(View->GC);
  gdk_draw_rectangle(View->GetDrawable(),View->GC,0,EncapRect.x*MagA/MagB,EncapRect.y*MagA/MagB,(EncapRect.width-1)*MagA/MagB,(EncapRect.height-1)*MagA/MagB);
}

void
TCADFramedText::Export(FILE *Fl,TExportFormat Format)
{
	DTP("TCADFramedText::Export()\n");

  switch(Format) {
    case EF_LATEX:
      int x,y;
      char *Text;
      Text = GetText();
      x = EncapRect.x;
      y = Sheet->EncapRect.height-EncapRect.y-EncapRect.height;
      fprintf(Fl,"\\put(%d,%d){\\framebox(%d,%d){%s}}\n",x,y,EncapRect.width,EncapRect.height,Text);
      free(Text);
      break;
    case EF_FIG:
      ExportFigFrame(Fl,XFigLineStyle());
      ExportFigText(Fl);
      break;
    case EF_EPS:
      ExportPSLineStyle(Fl);
      ExportPSFrame(Fl);
      ExportPSText(Fl);
			fputc('\n',Fl);
      break;
  }
}

void TCADFramedText::GetOffsets(int& TOff,int& BOff,int& LOff,int& ROff)
{
	DTP("TCADFramedText::GetOffsets()\n");

  TOff = TOP_OFFSET;
  BOff = BOTTOM_OFFSET;
  LOff = LEFT_OFFSET;
  ROff = RIGHT_OFFSET;
}

