/*******************\
*                   *
*  Line properties  *
*                   *
\*******************/

#include <math.h>
#include <gtk/gtk.h>
#include <string.h>
#include "lineproperties.h"
#include "object.h"
#include "sheet.h"

// Constants for line style
#define NUM_STYLES 6
static GdkLineStyle LineStyles[NUM_STYLES] = { GDK_LINE_SOLID, GDK_LINE_ON_OFF_DASH, GDK_LINE_ON_OFF_DASH, GDK_LINE_ON_OFF_DASH, GDK_LINE_ON_OFF_DASH, GDK_LINE_ON_OFF_DASH };
static char DashList[NUM_STYLES][8] = {
  { 9, 0, 0, 0, 0, 0, 0, 0 },
  { 1, 3, 0, 0, 0, 0, 0, 0 },
  { 2, 2, 0, 0, 0, 0, 0, 0 },
  { 4, 2, 1, 2, 0, 0, 0, 0 },
  { 4, 2, 1, 2, 1, 2, 0, 0 },
  { 4, 2, 1, 2, 1, 2, 1, 2 }
};

static int DashListLen[NUM_STYLES] = { 1, 2, 2, 4, 6, 8 };
static int DashList2Fig[NUM_STYLES] = { 0, 2, 1, 3, 4, 5 };

TCADLinePropertiesI::TCADLinePropertiesI()
{
	DTP("TCADLinePropertiesI::TCADLinePropertiesI()\n");

	LineStyle = 0;
	LineWidth = 1;
}

TCADLinePropertiesI::~TCADLinePropertiesI()
{
	DTP("TCADLinePropertiesI::~TCADLinePropertiesI()\n");

}

void
TCADLinePropertiesI::SaveLineProperties(TCADSaveStream *Fl)
{
	DTP("TCADLinePropertiesI::SaveLineProperties()\n");

  Fl->SaveInt("LineStyle",LineStyle);
  Fl->SaveInt("LineWidth",LineWidth);
}

char 
TCADLinePropertiesI::LoadProperty(TCADLoadStream *Fl, char *ID)
{
	DTP("TCADLinePropertiesI::LoadProperty()\n");

	if (strcasecmp(ID,"LineStyle") == 0) LineStyle = Fl->LoadInt();
	else if (strcasecmp(ID,"LineWidth") == 0) LineWidth = Fl->LoadInt();
  else return(FALSE);
}

/*
 *  Dialog routines
 */

void
TCADLinePropertiesI::DoDraw(GtkWidget *widget)
{
  int H = widget->allocation.height;
  int Num=0;
	int i;

	DTP("TCADLinePropertiesI::DoDraw()\n");

	for(i=0;i<NUM_STYLES;i++)
		if (widget == RB_LS[i]) {
			Num = i;
			break;
		}

	gdk_gc_set_line_attributes(GC,1,LineStyles[Num],GDK_CAP_NOT_LAST,GDK_JOIN_MITER);
	gdk_gc_set_dashes(GC,0,DashList[Num],DashListLen[Num]);
  gdk_draw_line(widget->window,GC,10,H/2,60,H/2);
}

static void
on_RB_draw(GtkWidget *widget, GdkRectangle *area, gpointer user_data)
{
	DTP("on_RB_draw()\n");

  area=area;

  ((TCADLinePropertiesI *)user_data)->DoDraw(widget);
}


static gboolean
on_RB_expose(GtkWidget *widget, GdkEventExpose  *event, gpointer user_data)
{
	DTP("on_RB_expose()\n");

  event = event;
  ((TCADLinePropertiesI *)user_data)->DoDraw(widget);
  return FALSE;
}

void
on_BCancel_pressed(GtkButton *button, gpointer user_data)
{
	DTP("on_BCancel_pressed()\n");

	button = button;

	((TCADLinePropertiesI *)user_data)->CloseDialog();
}

void
on_BOK_pressed(GtkButton *button, gpointer user_data)
{
	DTP("on_BOK_pressed()\n");

	((TCADLinePropertiesI *)user_data)->GetSettings();
	on_BCancel_pressed(button,user_data);
}

GtkWidget *
TCADLinePropertiesI::CreateDialog()
{
  GtkWidget *DLineProperties;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *frame4;
  GtkWidget *table1;
  GSList *table1_group = NULL;
  GtkWidget *frame5;
  GtkWidget *hbox2;
  GtkWidget *label1;
  GtkObject *SB_Width_adj;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox1;
  GtkWidget *BOK;
  GtkWidget *BCancel;
	GtkWidget *Res;
	int i,X,Y;

	DTP("TCADLineProperties::CreateDialog()\n");

  DLineProperties = gtk_dialog_new ();

  gtk_window_set_title (GTK_WINDOW (DLineProperties), "Line properties");
  gtk_window_set_policy (GTK_WINDOW (DLineProperties), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (DLineProperties)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);
  frame4 = gtk_frame_new ("Style");
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox2), frame4, TRUE, TRUE, 0);

  table1 = gtk_table_new (4, 3, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame4), table1);

	X = 0;
	Y = 0;
	for(i=0;i<NUM_STYLES;i++) {
	  Res = RB_LS[i] = gtk_radio_button_new_with_label (table1_group, "");
	  table1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (Res));
  	gtk_widget_show (Res);
	  gtk_table_attach (GTK_TABLE (table1), Res, X, X+1, Y, Y+1, (GtkAttachOptions) GTK_SHRINK, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
	  gtk_widget_set_usize (Res, 70, -1);
	  gtk_signal_connect_after (GTK_OBJECT (Res), "draw", GTK_SIGNAL_FUNC (on_RB_draw), this);
	  gtk_signal_connect_after (GTK_OBJECT (Res), "expose_event", GTK_SIGNAL_FUNC (on_RB_expose), this);
	  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (Res), (X==0)&&(Y==0));
	  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (Res), FALSE);

		X++;
		if (X == 3) {
			X = 0;
			Y++;
		}
	}

  frame5 = gtk_frame_new ("Width");
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox2), frame5, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (frame5), hbox2);

  label1 = gtk_label_new ("Line width:");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox2), label1, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);

  SB_Width_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  SB_Width = gtk_spin_button_new (GTK_ADJUSTMENT (SB_Width_adj), 1, 0);
  gtk_widget_show (SB_Width);
  gtk_box_pack_start (GTK_BOX (hbox2), SB_Width, TRUE, TRUE, 0);

  dialog_action_area1 = GTK_DIALOG (DLineProperties)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox1, TRUE, TRUE, 0);

  BOK = gtk_button_new_with_label ("OK");
  gtk_widget_show (BOK);
  gtk_box_pack_start (GTK_BOX (hbox1), BOK, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (BOK), "pressed", GTK_SIGNAL_FUNC (on_BOK_pressed), this);

  BCancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (BCancel);
  gtk_box_pack_start (GTK_BOX (hbox1), BCancel, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (BCancel), "pressed", GTK_SIGNAL_FUNC (on_BCancel_pressed), this);

	return DLineProperties;
}

void
TCADLinePropertiesI::ShowLineProperties(TCADSheet *Sheet)
{
  GdkColormap *ColorMap;

	DTP("TCADLinePropertiesI::ShowLineProperties()\n");

  GC = NULL;

	LP_Sheet = Sheet;

  Dialog = CreateDialog();

	// Setup widgets
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(RB_LS[LineStyle]),TRUE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(SB_Width),LineWidth);

  gtk_widget_show(Dialog);

  GC = gdk_gc_new(Dialog->window);

  // Get colors
  ColorMap = gdk_window_get_colormap(Dialog->window);
  gdk_color_white(ColorMap,&WhiteColor);
  gdk_color_black(ColorMap,&BlackColor);
  gdk_gc_set_background(GC,&WhiteColor);
  gdk_gc_set_foreground(GC,&BlackColor);

  gtk_grab_add(Dialog);
}

void 
TCADLinePropertiesI::CloseDialog()
{
	DTP("TCADLinePropertiesI::CloseDialog()\n");

  // Destroy GC
  if (GC != NULL) gdk_gc_destroy(GC);

	if (Dialog != NULL) {
	  // Make window non-modal
  	gtk_grab_remove(Dialog);

	  // Close window
  	gtk_widget_destroy(Dialog);
	}

	GC = NULL;
	Dialog = NULL;
}

void
TCADLinePropertiesI::GetSettings()
{
	int i;
	TCADObject *Obj;

	DTP("TCADLinePropertiesI::GetSettings()\n");

	for(i=0;i<NUM_STYLES;i++) {
		if (GTK_TOGGLE_BUTTON(RB_LS[i])->active) {
			LineStyle = i;
			break;
		}
	}

  LineWidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(SB_Width));

	// Refresh
	Obj = LP_Sheet->GetFirstSelected();
	while(Obj) {
		if (Obj->HasLineProperties()) {
			Obj->SetLineProperties(LineStyle,LineWidth);
		}
		Obj = Obj->NextSelected;
	}

	// Refresh
	LP_Sheet->DrawRect(NULL);
	LP_Sheet->Changed();
}

void
TCADLinePropertiesI::SetLineProperties(int Style,int Width)
{
	DTP("TCADLinePropertiesI::SetLineProperties()\n");

	LineStyle = Style;
	LineWidth = Width;
}

void
TCADLinePropertiesI::SetupGC(GdkGC *GC)
{
	DTP("TCADLinePropertiesI::SetupGC()\n");

	gdk_gc_set_line_attributes(GC,LineWidth,LineStyles[LineStyle],GDK_CAP_NOT_LAST,GDK_JOIN_MITER);
	if (LineStyle > 0) gdk_gc_set_dashes(GC,0,DashList[LineStyle],DashListLen[LineStyle]);
}

int
TCADLinePropertiesI::XFigLineStyle()
{
	DTP("TCADLinePropertiesI::XFigLineStyle()\n");

	return DashList2Fig[LineStyle];
}

void
TCADLinePropertiesI::ExportPSLineStyle(FILE *Fl)
{
	int i;

	DTP("TCADLinePropertiesI::ExportPSLineStyle()\n");

	if (LineStyles[LineStyle] == GDK_LINE_SOLID) fputs("[] 0 setdash\n",Fl);
	else {
		fputc('[',Fl);
		for(i=0;i<DashListLen[LineStyle];i++) {
			if (i > 0) fputc(' ',Fl);
			fprintf(Fl,"%d",DashList[LineStyle][i]);
		}
		fputs("] 0 setdash\n",Fl);
	}
}
