/*****************************\
*                             *
*  Implements TCADSimpleLine  *
*                             *
\*****************************/

#include <string.h>
#include <math.h>
#include "simpleline.h"
#include "view.h"
#include "sheet.h"

TCADSimpleLine::TCADSimpleLine(TCADSheet *Sheet,int x=0,int y=0) : TCADLine(Sheet,x,y)
{
	DTP("TCADSimpleLine::TCADSimpleLine()\n");

  RecomputeEncapRect();
  RecomputeRefRect();
}

TCADSimpleLine::~TCADSimpleLine()
{
	DTP("TCADSimpleLine::~TCADSimpleLine()\n");

}

void
TCADSimpleLine::Save(TCADSaveStream *Fl)
{
	DTP("TCADSimpleLine::Save()\n");

	if ((Fl->SaveOnlySelected) && !(Selected)) return;

  Fl->SaveObjectName("TCADSimpleLine");
  SaveProperties(Fl);
  Fl->SaveEndObject();
}

void
TCADSimpleLine::Draw(TCADView *View,GdkRegion *Region,int MagA,int MagB)
{
  GdkDrawable *Drawable;

//	DTP("TCADSimpleLine::Draw()\n");

  if (Region)
   if (gdk_region_rect_in(Region, &RefRect) == GDK_OVERLAP_RECTANGLE_OUT) return;

  Drawable = View->GetDrawable();
	SetupGC(View->GC);
  gdk_draw_line(Drawable,View->GC,First.x*MagA/MagB,First.y*MagA/MagB,Last.x*MagA/MagB,Last.y*MagA/MagB);

  // Draw arrows
  DrawArrow(Drawable,View->GC,&First,&Last,FirstArrow,FirstArrowSize,MagA,MagB);
  DrawArrow(Drawable,View->GC,&Last,&First,LastArrow,LastArrowSize,MagA,MagB);
}

TCADObject *
TCADSimpleLine::ButtonPressed(int x,int y)
{
	DTP("TCADSimpleLine::ButtonPressed()\n");

  if (TCADLine::ButtonPressed(x,y) == NULL) return(NULL);

  /* Compute distance */
  if (PointNearLine(First,Last,x,y)) return(this);
  else return(NULL);
}

void
TCADSimpleLine::Export(FILE *Fl,TExportFormat Format)
{
	GdkPoint P[2];

	DTP("TCADSimpleLine::Export()\n");

  switch(Format) {
    case EF_LATEX:
      int W,H;
      int x,y;
      x = EncapRect.x;
      y = Sheet->EncapRect.height-EncapRect.y;

      W = EncapRect.width;
      H = EncapRect.height;
      fprintf(Fl,"\\put(%d,%d){\\line(%d,%d){%f}}\n",x,y,W,-H,sqrt(W*W+H*H));
      break;
    case EF_FIG:
      ExportFigLine(Fl,2);
      fprintf(Fl,"\t%d %d  %d %d\n",First.x,First.y,Last.x,Last.y);
      break;
    case EF_EPS:
			P[0] = First;
			P[1] = Last;
      ExportPSLine(Fl,P,2);
			break;
  }
}

void
TCADSimpleLine::RecomputeEncapRect()
{
	DTP("TCADSimpleLine::RecomputeEncapRect()\n");

  EncapRect.x = MIN(First.x,Last.x);
  EncapRect.y = MIN(First.y,Last.y);
  EncapRect.width = ABS(First.x-Last.x);
  EncapRect.height = ABS(First.y-Last.y);
}

gboolean
TCADSimpleLine::CanAttach(int& x,int& y,TAttach *Attach)
{
	DTP("TCADSimpleLine::CanAttach()\n");

  return(FALSE);
}


