/**********************\
*                      *
*  Read/write streams  *
*                      *
\**********************/

/*
 * Known bugs:
 *
 * - Saving too many embded objects will cause incorrect indentation
 * - Very long strings will be cut at load
 */

#include "streams.h"
#include "debug.h"


/******************************************************************************/


/*
 *  TCADMemory
 */
void
TCADMemory::EnsureSize(int NeededSize)
{
//	DTP("TCADMemory::EnsureSize()\n");

	if (Size >= NeededSize) return;

	while(Size < NeededSize) Size += Step;

	Memory = realloc(Memory,Size);
}


/******************************************************************************/


/*
 * TCADFileSaveStream
 */

TCADFileSaveStream::TCADFileSaveStream(char *FileName,char &Success)
{
	DTP("TCADFileSaveStream::TCADFileSaveStream()\n");

  Depth = 0;
  Spaces[0] = 0;
  Fl = fopen(FileName,"w");
  Success = (Fl != 0);
}

TCADFileSaveStream::~TCADFileSaveStream()
{
	DTP("TCADFileSaveStream::~TCADFileSaveStream()\n");

  if (Fl != 0) fclose(Fl);
}

void
TCADFileSaveStream::SaveInt(char *Name,int Data)
{
//	DTP("TCADFileSaveStream::SaveInt()\n");

  if (Fl == 0) return;

  fprintf(Fl,"%s%s=%d\n",Spaces,Name,Data);
}

void
TCADFileSaveStream::SaveString(char *Name,char *Data)
{
//	DTP("TCADFileSaveStream::SaveString()\n");

  if (Fl == 0) return;

  fprintf(Fl,"%s%s=",Spaces,Name);

  // Convert
  while(*Data) {
    if ((*Data == '^') || (*Data < 32)) {
			fputc('^',Fl);
			fputc(*Data+64,Fl);
    }
    else fputc(*Data,Fl);

    Data++;
  }
  fputc(10,Fl);
}

void
TCADFileSaveStream::SaveObjectName(char *Name)
{
//	DTP("TCADFileSaveStream::SaveObjectName()\n");

  if (Fl == 0) return;

  fprintf(Fl,"%s%s {\n",Spaces,Name);
  if (Depth < MAXDEPTH) {
    Spaces[Depth++] = ' ';
    Spaces[Depth++] = ' ';
    Spaces[Depth] = 0;
  }
}

void
TCADFileSaveStream::SaveEndObject()
{
//	DTP("TCADFileSaveStream::SaveEndObject()\n");

  if (Fl == 0) return;

  Depth -= 2;
  if (Depth < 0) Depth = 0;
  Spaces[Depth] = 0;

  fprintf(Fl,"%s}\n",Spaces);
}


/******************************************************************************/


/*
 * TCADMemorySaveStream
 */

void
TCADMemorySaveStream::SaveInt(char *Name,int Data)
{
	char *P;
//	DTP("TCADMemorySaveStream::SaveInt()\n");

	mem->EnsureSize(off+strlen(Name)+20);
	P = (char *)mem->Memory;
	P += off;

  sprintf(P,"%s=%d\n",Name,Data);
	off += strlen(P);
}

void
TCADMemorySaveStream::SaveString(char *Name,char *Data)
{
	char *P;
	int T;

//	DTP("TCADMemorySaveStream::SaveString()\n");

	// We need at most two characters per data character
	mem->EnsureSize(off+strlen(Name)+strlen(Data)*2+5);
	P = (char *)mem->Memory;
	P += off;

  sprintf(P,"%s=",Name);
	T = strlen(Name)+1;
	P += T;
	off += T;

  // Convert
  while(*Data) {
    if ((*Data == '^') || (*Data < 32)) {
			*P++ = '^';
			*P++ = *Data+64;
			off += 2;
    }
    else {
			*P++ = *Data;
			off++;
		}

    Data++;
  }
  *P++ = 10;
	off++;
	*P = 0;
}

void
TCADMemorySaveStream::SaveObjectName(char *Name)
{
	int L = strlen(Name);

//	DTP("TCADMemorySaveStream::SaveObjectName()\n");

	mem->EnsureSize(off+L+5);
	sprintf(((char *)mem->Memory)+off,"%s {\n",Name);
	off += L+3;
}

void
TCADMemorySaveStream::SaveEndObject()
{
//	DTP("TCADMemorySaveStream::SaveEndObject()\n");

	mem->EnsureSize(off+5);
	strcpy(((char *)mem->Memory)+off,"}\n");
	off += 2;
}

void
TCADMemorySaveStream::SaveToFile(char *FileName)
{
	FILE *Fl;

	DTP("TCADMemorySaveStream::SaveToFile()\n");

	Fl = fopen(FileName,"w");
	if (!Fl) return;

	fwrite(mem->Memory,1,off,Fl);
	fclose(Fl);
}

/******************************************************************************/

/*
 *  TCADLoadStream
 */

char *
TCADLoadStream::ReadID()
{
  int a;
  int IDOff = 0;

	DTP("TCADLoadStream::ReadID()\n");

  WasLF = 0;

  for(;;) {
    do {
      a = NextChar();
    } while(a == 13); // Ignore CRs
    
    if (a < 0) {
      WasLF = 1;
      Buffer[0] = 0;
      ID[IDOff] = 0;
      return(ID);
    }

    if ((a == 10) || (a == '=')) {
      WasLF = (a == 10);
      ID[IDOff] = 0;
      return(ID);
    }

    if ((a != ' ') && (a != '{') && (a != '}'))  // Ignore these characters
     if (IDOff < MAXIDLEN) ID[IDOff++] = a;
  }
}

void
TCADLoadStream::ReadROL()
{
  char a;
  int BufOff;
  
	DTP("TCADLoadStream::ReadROL()\n");

  BufOff = 0;
  for(;;) {
    do {
      a = NextChar();
    } while(a == 13);
    
    if ((a == -1) || (a == 10)) break;

    if (BufOff < BUFSIZE) Buffer[BufOff++] = a;
  }
  
  Buffer[BufOff] = 0;
}

char *
TCADLoadStream::LoadString()
{
  char *SrcPtr,*DstPtr;

	DTP("TCADLoadStream::LoadString()\n");

	if (WasLF) {
		Buffer[0] = 0;
		return(Buffer);
	}

	ReadROL();  // Read rest of line to buffer

	// Convert text
	SrcPtr = Buffer;
	DstPtr = Buffer;
	while(*SrcPtr) {
		if (*SrcPtr == '^') {
			SrcPtr++;
			if (*SrcPtr == 0) {
				*DstPtr++ = '^';
				break;
      }
      else *DstPtr++ = *SrcPtr++ - 64;
    }
    else *DstPtr++ = *SrcPtr++;
  }
  *DstPtr = 0;

  return(Buffer);
}

int
TCADLoadStream::LoadInt()
{
  int Result;

	DTP("TCADLoadStream::LoadInt()\n");

  if (WasLF) return(0);

  ReadROL();  // Read rest of line to buffer

  Result = 0;
  sscanf(Buffer,"%d",&Result);
  return(Result);
}
      
void
TCADLoadStream::Skip()
{
	DTP("TCADLoadStream::Skip()\n");

  if (ID[0] == 0) return;  // Don't skip EOF

  if (WasLF) {
    int Depth=1;
    
    // Skip object
    while(Depth > 0) {
      ReadID();
      if (ID[0] == 0) Depth--;  // End of object
      else if (WasLF) Depth++;  // Another object
      else LoadString();        // Skip value
    }
  }
  else LoadString(); // Skip value
}

void
TCADLoadStream::UniqTranslated(unsigned long Old,unsigned long New)
{
	DTP("TCADLoadStream::UniqTranslated()\n");

	if (UniqTable == NULL) UniqTable = g_hash_table_new(g_direct_hash,g_direct_equal);

	g_hash_table_insert(UniqTable,(gpointer)Old,(gpointer)New);
}

unsigned long
TCADLoadStream::TranslateUniq(unsigned long Old)
{
	DTP("TCADLoadStream::TranslateString()\n");

	if (UniqTable == NULL) return(0);
	if (Old == 0) return(0);

	return((int)g_hash_table_lookup(UniqTable,gpointer(Old)));
}

/******************************************************************************/

/*
 * TCADFileLoadStream
 */

TCADFileLoadStream::TCADFileLoadStream(char *FileName,char &Success)
{
	DTP("TCADFileLoadStream::TCADFileLoadStream()\n");

  ID[0] = 0;
  WasLF = 1;
  Fl = fopen(FileName,"r");
  Success = (Fl != 0);
}

TCADFileLoadStream::~TCADFileLoadStream()
{
	DTP("TCADFileLoadStream::~TCADFileLoadStream()\n");

  if (Fl != 0) fclose(Fl);
}

/******************************************************************************/
