/***********************\
*                       *
*  Implements TCADText  *
*                       *
\***********************/

#include <string.h>
#include <stdlib.h>
#include "text.h"
#include "view.h"

TCADText::TCADText(TCADSheet *Sheet) : TCADTextObject(Sheet)
{
	DTP("TCADText::TCADText()\n");

  Text = NULL;
  Lines = 0;
}

TCADText::~TCADText()
{
	DTP("TCADText::~TCADText()\n");

  if (Text != NULL) free(Text);
}

void
TCADText::Save(TCADSaveStream *Fl)
{
	DTP("TCADText::Save()\n");

	if ((Fl->SaveOnlySelected) && !(Selected)) return;

  Fl->SaveObjectName("TCADText");
  SaveProperties(Fl);
  Fl->SaveEndObject();
}

void
TCADText::SaveProperties(TCADSaveStream *Fl)
{
  char *Tmp;

	DTP("TCADText::SaveProperties()\n");

  TCADObject::SaveProperties(Fl);

  SaveTextProperties(Fl);
  if (Text != NULL) {
    Tmp = GetText();
    Fl->SaveString("Text",Tmp);
    free(Tmp);
  }
}

char
TCADText::LoadProperty(TCADLoadStream *Fl,char *ID)
{
	DTP("TCADText::LoadProperty()\n");

  if (strcasecmp(ID,"Text") == 0) SetText(Fl->LoadString());
  else if (TCADTextPropertiesI::LoadProperty(Fl,ID)) ;
  else return TCADObject::LoadProperty(Fl,ID);

  return(TRUE);
}

gchar *
TCADText::GetText()
{
  char *Result,*TmpPtr;
  int i;

	DTP("TCADText::GetText()\n");

  if (Text == 0) {
    Result = (char *)malloc(1);
    *Result = 0;
  }
  else {
    Result = (char *)malloc(TextLength);
    memcpy(Result,Text,TextLength);

    // Insert LFs
    TmpPtr = Result;
    for(i=1;i<TextLength;i++)
     if (*(TmpPtr++) == 0)
      *(TmpPtr-1) = 10;
  }

  return(Result);
}

void
TCADText::SetText(gchar *Text)
{
  char *TmpPtr;

	DTP("TCADText::SetText()\n");

  if (this->Text != NULL) free(this->Text);
  this->Text = strdup(Text);

  /* Count lines and break text to lines */
  Lines = 1;
  TextLength = 1;
  TmpPtr = this->Text;
  while(*TmpPtr) {
    if (*TmpPtr == 10) {
      *TmpPtr = 0;
      Lines++;
    }
    TmpPtr++;
    TextLength++;
  }
}

void
TCADText::GetOffsets(int& TOff,int& BOff,int& LOff,int& ROff)
{
	DTP("TCADText::GetOffsets()\n");

  TOff = 0;
  BOff = 0;
  LOff = 0;
  ROff = 0;
}

void
TCADText::Draw(TCADView *View,GdkRegion *Region,int MagA,int MagB)
{
  GdkGC *GC;
  int TOff,BOff,LOff,ROff; // Offsets
	GdkRectangle Rect;
	int T;

//	DTP("TCADText::Draw()\n");

  if (Text == NULL) return;

  if (Region)
   if (gdk_region_rect_in(Region, &EncapRect) == GDK_OVERLAP_RECTANGLE_OUT) return;

  GetOffsets(TOff,BOff,LOff,ROff);

	Rect = EncapRect;
	Rect.x += LOff;
	Rect.y += TOff;

	T = TOff+BOff;
	if (T >= Rect.width) return;
	Rect.width -= T;

	T = LOff+ROff;
	if (T >= Rect.height) return;
	Rect.height -= (LOff+ROff);

	DrawText(*(View->GetDrawable()),*(View->GC),Text,Lines,Rect,MagA,MagB);
}

void
TCADText::Select(GdkRectangle *RefreshRect)
{
	DTP("TCADText::Select()\n");

  TCADObject::Select(RefreshRect);
   UpdateHandlePos();
}

void
TCADText::Export(FILE *Fl,TExportFormat Format)
{
	DTP("TCADText::Export()\n");

  switch(Format) {
    case EF_LATEX:
      int x,y;
      char *Text;
      Text = GetText();
      x = EncapRect.x;
      y = Sheet->EncapRect.height-EncapRect.y-EncapRect.height;
      fprintf(Fl,"\\put(%d,%d){\\makebox(%d,%d){%s}}\n",x,y,EncapRect.width,EncapRect.height,Text);      free(Text);
      break;
    case EF_FIG:
      ExportFigText(Fl);
      break;
    case EF_EPS:
      ExportPSText(Fl);
			fputc('\n',Fl);
      break;
  }
}

void
TCADText::ExportFigText(FILE *Fl)
{
  unsigned char *Text,*TmpPtr;
  int X,Y;
  int Align=0;
	int FigFont,FigFontFlags;

	DTP("TCADText::ExportFigText()\n");

	GetTextXY(X,Y);

  Text = (unsigned char *)GetText();
  TmpPtr = Text;

	if (TextHAlign == AL_LEFT) Align=0;
	else if (TextHAlign == AL_HCENTER) Align=1;
	else if (TextHAlign == AL_RIGHT) Align=2;

  // Create one text object for every line
	GetFigFont(FigFont,FigFontFlags);
  while(*TmpPtr) {
    fprintf(Fl,"4 %d %d %d %d %d %d %f %d %d %d %d %d ",
     Align,
     0,        // Color
     0,        // Depth
     0,        // Pen style
     FigFont,        // Font
     FontSize, // Fontsize
     (float)0, // Angle
     FigFontFlags,        // Font flags
     EncapRect.height,
     EncapRect.width,
     X,
     Y
     );

    while(*TmpPtr) {
      if (*TmpPtr == 10) break;

      if (*TmpPtr < 128) fputc(*TmpPtr,Fl);
      else fprintf(Fl,"\\%03o",*TmpPtr);
      TmpPtr++;
    }
    fputs("\\001\n",Fl);
    if (*TmpPtr == 10) TmpPtr++;

    Y += FontSize;
  }
  
  free(Text);
}

void
TCADText::ExportPSText(FILE *Fl)
{
	int X,Y;
	unsigned char *Text,*P1,*Font;
	char *Cmd = "LShow";

	DTP("TCADText::ExportPSText()\n");

	GetTextXY(X,Y);
	Y-=2;

	/* Export font */
	// Find font name
	P1 = (unsigned char *)FontName;

	if (TextHAlign == AL_LEFT) Cmd = "LShow";
	else if (TextHAlign == AL_HCENTER) Cmd = "CShow";
	else if (TextHAlign == AL_RIGHT) Cmd = "RShow";

  Text = (unsigned char *)GetText();
  P1 = Text;
	ExportEPSFont(Fl);
	while(*P1) {
		fprintf(Fl,"%d %d (",X,Sheet->EncapRect.height-Y);
		while((*P1) && (*P1 != 10)) {
			if ((*P1 == '(') || (*P1 == ')')) fputc('\\',Fl);
			fputc(*P1,Fl);
			P1++;
		}
		if (*P1) P1++;
		fprintf(Fl,") %s\n",Cmd);

		Y += FontSize;
	}
	
	free(Text);
}

void
TCADText::GetTextXY(int &X,int &Y,gboolean ReverseY)
{
  int TOff,BOff,LOff,ROff;
	GetOffsets(TOff,BOff,LOff,ROff);
	TVAlign va;

	DTP("TCADText::GetTextXY()\n");

  X = EncapRect.x+LOff;
  switch(TextHAlign) {
    case AL_HCENTER:
      X += (EncapRect.width-LOff-ROff)/2;
      break;
    case AL_RIGHT: 
      X += (EncapRect.width-LOff-ROff);
      break;
  }

  Y = EncapRect.y+TOff+FontSize;
	va = TextVAlign;
	if (ReverseY) {
		if (va == AL_TOP) va = AL_BOTTOM;
		else if (va = AL_BOTTOM) va = AL_TOP;
	}
  switch(va) {
    case AL_VCENTER:
      Y += (EncapRect.height-TOff-BOff)/2;
      Y -= (Lines*FontSize)/2;
      break;
    case AL_BOTTOM: 
      Y += (EncapRect.height-BOff-TOff);
      Y -= (Lines*FontSize);
      break;
  }
}
