/*******************\
*                   *
*  Text properties  *
*                   *
\*******************/

#include <string.h>
#include <gtk/gtk.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include "textproperties.h"
#include "textobject.h"
#include "object.h"
#include "fontdialog.h"
#include "main.h"

	/* Conversion tables for EPS */

  // Font family conversion table.
  static char *FontCT[] = {
"courier","fixed","terminal","lucidatypewriter", "/Courier",
"helvetica","arial","lucida","clean", "/Helvetica",
"*", "/Times"
};

  // Weight conversion table
  static char *WeightCT[] = {
"bold","demibold", "-Bold",
"*", ""
};

  // Width conversion table
  static char *WidthCT[] = {
"condensed","semicondensed", "-Condensed",
"*", ""
};

  // Style conversion table
  static char *StyleCT[] = {
"i","o","s", "-Italic",
"*", "-Roman"
};

	/* Conversion tables for XFig - now the same as for PS */
  static char *FontLCT[] = {
"courier","fixed","terminal","lucidatypewriter", "/Courier",
"helvetica","arial","lucida","clean", "/Helvetica",
"*", "/Times"
};
  static char *WeightLCT[] = {
"bold","demibold", "-Bold",
"*", ""
};
  static char *StyleLCT[] = {
"i","o","s", "-Italic",
"*", "-Roman"
};

static TCADObject *Selected;
static GtkWidget *Dialog;
static GtkWidget *RBBR;
static GtkWidget *RBB;
static GtkWidget *RBBL;
static GtkWidget *RBR;
static GtkWidget *RBC;
static GtkWidget *RBL;
static GtkWidget *RBTR;
static GtkWidget *RBT;
static GtkWidget *RBTL;

static TVAlign TextVAlign;
static THAlign TextHAlign;
static char DFontName[PATH_MAX];

static char DefaultFontName[] = "-*-times-medium-r-normal--%d-*-*-*-p-*-iso8859-1";

TCADTextPropertiesI::TCADTextPropertiesI()
{
  TextHAlign = AL_HCENTER;
  TextVAlign = AL_VCENTER;

	DTP("TCADTextPropertiesI::TCADTextPropertiesI()\n");

	FNSize = strlen(DefaultFontName)+10;
	FontName = (char *)malloc(FNSize);
	strcpy(FontName,DefaultFontName);
	FontSize = 12;
	FontNameBuf = (char *)malloc(FNSize);
}

TCADTextPropertiesI::~TCADTextPropertiesI()
{
	DTP("TCADTextPropertiesI::~TCADTextPropertiesI()\n");

	free(FontName);
	free(FontNameBuf);
}

void
SetFont(char *FontName,void *UserData)
{
	DTP("SetFont()\n");

	strcpy(DFontName,FontName);
}

static void
on_BCancel_clicked(GtkButton *button, gpointer user_data)
{
	DTP("on_BCancel_clicked()\n");

  button = button;
  user_data = user_data;

  // Make window non-modal
  gtk_grab_remove(Dialog);

  // Close window
  gtk_widget_destroy(Dialog);
}

static void
on_BSetFont_clicked(GtkButton *button, gpointer user_data)
{
	DTP("on_BSetFont_clicked()\n");

  button = button;
  user_data = user_data;

	FontDialog(DFontName,SetFont,user_data);
}

static void
GetSettings()
{
	DTP("GetSettings()\n");

  if (GTK_TOGGLE_BUTTON(RBBR)->active) {
    TextVAlign = AL_BOTTOM;
    TextHAlign = AL_RIGHT;
  }
  else if (GTK_TOGGLE_BUTTON(RBB)->active) {
    TextVAlign = AL_BOTTOM;
    TextHAlign = AL_HCENTER;
  }
  else if (GTK_TOGGLE_BUTTON(RBBL)->active) {
    TextVAlign = AL_BOTTOM;
    TextHAlign = AL_LEFT;
  }
  else if (GTK_TOGGLE_BUTTON(RBR)->active) {
    TextVAlign = AL_VCENTER;
    TextHAlign = AL_RIGHT;
  }
  else if (GTK_TOGGLE_BUTTON(RBC)->active) {
    TextVAlign = AL_VCENTER;
    TextHAlign = AL_HCENTER;
  }
  else if (GTK_TOGGLE_BUTTON(RBL)->active) {
    TextVAlign = AL_VCENTER;
    TextHAlign = AL_LEFT;
  }
  else if (GTK_TOGGLE_BUTTON(RBTR)->active) {
    TextVAlign = AL_TOP;
    TextHAlign = AL_RIGHT;
  }
  else if (GTK_TOGGLE_BUTTON(RBT)->active) {
    TextVAlign = AL_TOP;
    TextHAlign = AL_HCENTER;
  }
  else if (GTK_TOGGLE_BUTTON(RBTL)->active) {
    TextVAlign = AL_TOP;
    TextHAlign = AL_LEFT;
  }
}

static void
on_BOK_clicked (GtkButton *button, gpointer user_data)
{
	TCADObject *FS;

	DTP("on_BOK_clicked()\n");

  // Get settings
  GetSettings();

  // Close window
  on_BCancel_clicked(button,user_data);

  // Refresh line arrows
	FS = Selected;
  while(Selected != 0) {
    if (Selected->HasText()) ((TCADTextObject *)Selected)->SetTextProperties(TextVAlign,TextHAlign,DFontName);
    Selected = Selected->NextSelected;
  }

	if (FS != NULL) FS->GetSheet()->Changed();
}


static GtkWidget*
CreateDialog(gpointer UserData)
{
  GtkWidget *DTextProperties;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *FAlign;
  GtkWidget *table2;
  GSList *table2_group = NULL;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox1;
  GtkWidget *BOK;
  GtkWidget *BApply;
  GtkWidget *BCancel;
  GtkWidget *BSetFont;

	DTP("CreateDialog()\n");

  DTextProperties = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "DTextProperties", DTextProperties);
  gtk_window_set_title (GTK_WINDOW (DTextProperties), "Text properties");
  gtk_window_set_policy (GTK_WINDOW (DTextProperties), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (DTextProperties)->vbox;
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  FAlign = gtk_frame_new ("Alignment");
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "FAlign", FAlign);
  gtk_widget_show (FAlign);
  gtk_box_pack_start (GTK_BOX (vbox1), FAlign, TRUE, TRUE, 0);

  table2 = gtk_table_new (3, 3, TRUE);
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "table2", table2);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (FAlign), table2);

  RBBR = gtk_radio_button_new_with_label (table2_group, "Bottom right");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBBR));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBBR", RBBR);
  gtk_widget_show (RBBR);
  gtk_table_attach (GTK_TABLE (table2), RBBR, 2, 3, 2, 3,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBBR), FALSE);

  RBB = gtk_radio_button_new_with_label (table2_group, "Bottom");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBB));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBB", RBB);
  gtk_widget_show (RBB);
  gtk_table_attach (GTK_TABLE (table2), RBB, 1, 2, 2, 3,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBB), FALSE);

  RBBL = gtk_radio_button_new_with_label (table2_group, "Bottom left");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBBL));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBBL", RBBL);
  gtk_widget_show (RBBL);
  gtk_table_attach (GTK_TABLE (table2), RBBL, 0, 1, 2, 3,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBBL), FALSE);

  RBR = gtk_radio_button_new_with_label (table2_group, "Right");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBR));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBR", RBR);
  gtk_widget_show (RBR);
  gtk_table_attach (GTK_TABLE (table2), RBR, 2, 3, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBR), FALSE);

  RBC = gtk_radio_button_new_with_label (table2_group, "Center");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBC));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBC", RBC);
  gtk_widget_show (RBC);
  gtk_table_attach (GTK_TABLE (table2), RBC, 1, 2, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBC), FALSE);

  RBL = gtk_radio_button_new_with_label (table2_group, "Left");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBL));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBL", RBL);
  gtk_widget_show (RBL);
  gtk_table_attach (GTK_TABLE (table2), RBL, 0, 1, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBL), FALSE);

  RBTR = gtk_radio_button_new_with_label (table2_group, "Top right");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBTR));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBTR", RBTR);
  gtk_widget_show (RBTR);
  gtk_table_attach (GTK_TABLE (table2), RBTR, 2, 3, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBTR), FALSE);
  RBT = gtk_radio_button_new_with_label (table2_group, "Top");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBT));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBT", RBT);
  gtk_widget_show (RBT);
  gtk_table_attach (GTK_TABLE (table2), RBT, 1, 2, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBT), FALSE);

  RBTL = gtk_radio_button_new_with_label (table2_group, "Top left");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (RBTL));
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "RBTL", RBTL);
  gtk_widget_show (RBTL);
  gtk_table_attach (GTK_TABLE (table2), RBTL, 0, 1, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (RBTL), FALSE);


  BSetFont = gtk_button_new_with_label ("Set font");
	gtk_widget_show(BSetFont);
	gtk_container_add(GTK_CONTAINER(vbox1),BSetFont);
  gtk_signal_connect (GTK_OBJECT (BSetFont), "clicked",
                      GTK_SIGNAL_FUNC (on_BSetFont_clicked),SetFont);

  dialog_action_area1 = GTK_DIALOG (DTextProperties)->action_area;
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox1 = gtk_hbox_new (TRUE, 5);
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox1, TRUE, TRUE, 0);

  BOK = gtk_button_new_with_label ("OK");
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "BOK", BOK);
  gtk_widget_show (BOK);
  gtk_box_pack_start (GTK_BOX (hbox1), BOK, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (BOK), "clicked",
                      GTK_SIGNAL_FUNC (on_BOK_clicked),NULL);
						

  BApply = gtk_button_new_with_label ("Apply");
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "BApply", BApply);
  BCancel = gtk_button_new_with_label ("Cancel");
  gtk_object_set_data (GTK_OBJECT (DTextProperties), "BCancel", BCancel);
  gtk_widget_show (BCancel);
  gtk_box_pack_start (GTK_BOX (hbox1), BCancel, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (BCancel), "clicked",
                      GTK_SIGNAL_FUNC (on_BCancel_clicked),NULL);

  return DTextProperties;
}


void
TCADTextPropertiesI::ShowTextProperties(TCADObject *Obj)
{
  GtkWidget *DefB = NULL;

	DTP("TCADTextPropertiesI::ShowTextProperties()\n");

  Selected = Obj;

  Dialog = CreateDialog(this);

  // Set button which is on
  switch(TextHAlign) {
    case AL_LEFT:
      switch(TextVAlign) {
        case AL_TOP:
	  DefB = RBTL;
	  break;
        case AL_VCENTER:
	  DefB = RBL;
	  break;
        case AL_BOTTOM:
	  DefB = RBBL;
	  break;
      }
      break;
    case AL_HCENTER:
      switch(TextVAlign) {
        case AL_TOP:
	  DefB = RBT;
	  break;
        case AL_VCENTER:
	  DefB = RBC;
	  break;
        case AL_BOTTOM:
	  DefB = RBB;
	  break;
      }
      break;
    case AL_RIGHT:
      switch(TextVAlign) {
        case AL_TOP:
	  DefB = RBTR;
	  break;
        case AL_VCENTER:
	  DefB = RBR;
	  break;
        case AL_BOTTOM:
	  DefB = RBBR;
	  break;
      }
      break;
  }
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (DefB), TRUE);

	sprintf(DFontName,FontName,FontSize);

  gtk_widget_show(Dialog);
  gtk_grab_add(Dialog);
}

void
TCADTextPropertiesI::SaveTextProperties(TCADSaveStream *Fl)
{
	DTP("TCADTextPropertiesI::SaveTextProperties()\n");

  Fl->SaveInt("TextHAlign",TextHAlign);
  Fl->SaveInt("TextVAlign",TextVAlign);
  Fl->SaveString("FontName",FontName);
  Fl->SaveInt("FontSize",FontSize);
}

char
TCADTextPropertiesI::LoadProperty(TCADLoadStream *Fl,char *ID)
{
	char *S;

	DTP("TCADTextPropertiesI::LoadProperty()\n");

  if (strcasecmp("TextHAlign",ID) == 0) TextHAlign = (THAlign)Fl->LoadInt();
  else if (strcasecmp("TextVAlign",ID) == 0) TextVAlign = (TVAlign)Fl->LoadInt();
  else if (strcasecmp("FontName",ID) == 0) {
		S = Fl->LoadString();
		FNSize = strlen(S)+10;
		FontName = (char *)realloc(FontName,FNSize);
		strcpy(FontName,S);
		FontNameBuf = (char *)realloc(FontNameBuf,FNSize);
	}
  else if (strcasecmp("FontSize",ID) == 0) FontSize = Fl->LoadInt();
  else return(0);

  return(1);
}

void
TCADTextPropertiesI::DrawText(GdkDrawable &Drawable,GdkGC &GCtx,char *Text,int Lines,GdkRectangle &Rect,int MagA,int MagB)
{
	GdkGC *GC;
	int CH,XOff,YOff,i;
	char *TmpPtr;
	int RealSize;
	int DrawShadow,SW;
	GdkFont *TextFont;
	GdkRectangle Rect2;

	DTP("TCADTextPropertiesI::DrawText()\n");

  GC = gdk_gc_new(&Drawable);

  // Copy GC and limit drawing region to object size
  gdk_gc_copy(GC,&GCtx);
	Rect2 = Rect;
	Rect2.x = Rect2.x*MagA/MagB;
	Rect2.y = Rect2.y*MagA/MagB;
	Rect2.width = Rect2.width*MagA/MagB;
	Rect2.height = Rect2.height*MagA/MagB;
  gdk_gc_set_clip_rectangle(GC,&Rect2);

	/* Get font */
	RealSize = FontSize*MagA/MagB;
	if (RealSize <= 0) RealSize=1;
	DrawShadow = (RealSize < 8);

	if (DrawShadow) {
//		gdk_gc_set_fill(GC,GDK_TILED);
		sprintf(FontNameBuf,FontName,FontSize);
	}
	else sprintf(FontNameBuf,FontName,RealSize);

	TextFont = System->FontCache.GetFont(FontNameBuf);
  CH = gdk_text_height(TextFont,"Hello",5)+3;
	if (!DrawShadow) CH = CH*MagB/MagA;

  /* Compute YOff */
  YOff = Rect.y+CH;
  switch(TextVAlign) {
    case AL_VCENTER:
      YOff = Rect.y+CH+(Rect.height-Lines*CH) / 2;
      break;
    case AL_BOTTOM:
      YOff = Rect.y+CH+Rect.height-Lines*CH;
      break;
    default: ;
  }

  /* Draw */
  TmpPtr = Text;
  for(i=0;i<Lines;i++) {
    XOff = Rect.x;
		if (DrawShadow || (TextHAlign != AL_LEFT)) SW = gdk_string_width(TextFont,TmpPtr);
		if (!DrawShadow) SW = SW*MagB/MagA;
    switch(TextHAlign) {
      case AL_HCENTER:
        XOff = Rect.x+(Rect.width-SW) / 2;
        break;
      case AL_RIGHT:
        XOff = Rect.x+Rect.width-SW;
        break;
      default: ;
    }

		if (DrawShadow) gdk_draw_rectangle(&Drawable,GC,true,XOff*MagA/MagB+1,(YOff-CH)*MagA/MagB+1,SW*MagA/MagB,CH*MagA/MagB - 1);
		else gdk_draw_string(&Drawable,TextFont,GC,XOff*MagA/MagB,YOff*MagA/MagB,TmpPtr);
    while(*TmpPtr++);  // Advance to next line
    YOff += CH;
  }

  gdk_gc_destroy(GC);
}

void
TCADTextPropertiesI::GetFontProperties(FontProperties &fp)
{
	char *P;
	int i;

	DTP("TCADTextPropertiesI::GetFontProperties()\n");

	strcpy(FontNameBuf,FontName);
	P = FontNameBuf;
	for(i=0;i<FPI_NUMBEROF;i++) {
		fp.Prop[i] = P;
		while(*P && (*P != '-')) P++;
		if (*P) *P++ = 0;
	}
}

void
TCADTextPropertiesI::GetFigFont(int &FigFont,int &FigFontStyle)
{
	FontProperties fp;
	char Family[20],Weight[20],Style[20];

	DTP("TCADTextPropertiesI::GetFigFont()\n");

	GetFontProperties(fp);

	// Convert font family
	strcpy(Family,ConvertFontProperty(fp.Prop[FPI_FAMILY],&FontLCT[0]));


	// Convert weight
	strcpy(Weight,ConvertFontProperty(fp.Prop[FPI_WEIGHT],&WeightLCT[0]));

	// Convert style
	strcpy(Style,ConvertFontProperty(fp.Prop[FPI_SLANT],&StyleCT[0]));

	FigFontStyle=0;
	FigFont=0;

	if (strcmp(Family,"/Courier") == 0) {
		FigFont=5;
		return;
	}

	if (strcmp(Style,"-Italic") == 0) {
		FigFont=3;
		return;
	}

	if (strcmp(Weight,"-Bold") == 0) {
		FigFont=2;
		return;
	}

	if (strcmp(Family,"/Helvetica") == 0) {
		FigFont=4;
		return;
	}

	FigFont=1;
	return;
}

const char *
TCADTextPropertiesI::ConvertFontProperty(const char *Property,char **Table)
{
	int Found,i;

	DTP("TCADTextPropertiesI::ConvertFontProperty()\n");

	Found=0;
	for(i=0;;i++) {
		if (!isalpha(Table[i][0]) && (Table[i][0] != '*')) {
			if (Found) return(Table[i]);
		}
		else {
			if (Table[i][0] == '*') Found=1;
			else Found |= (strncasecmp(Property,Table[i],strlen(Table[i])) == 0);
		}
	}
}


void
TCADTextPropertiesI::ExportEPSFont(FILE *Fl)
{
	FontProperties fp;

	DTP("TCADTextPropertiesI::ExportEPSFont()\n");

	GetFontProperties(fp);
	
	// Convert font family
	fputs(ConvertFontProperty(fp.Prop[FPI_FAMILY],&FontCT[0]),Fl);

	// Convert weight
	fputs(ConvertFontProperty(fp.Prop[FPI_WEIGHT],&WeightCT[0]),Fl);

	// Convert width
	fputs(ConvertFontProperty(fp.Prop[FPI_SETWIDTH],&WidthCT[0]),Fl);

	// Convert style
	fputs(ConvertFontProperty(fp.Prop[FPI_SLANT],&StyleCT[0]),Fl);

	// The rest...
	fprintf(Fl," findfont\n%d scalefont\nsetfont\n\n",FontSize);
}

