/*****************\
*                 *
*  View of sheet  *
*                 *
\*****************/

#ifndef GTKFIG_VIEW_H
#define GTKFIG_VIEW_H

#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "handle.h"

class TCADSheet;

/*
 * Ruler settings
 */
typedef struct {
  gboolean Visible;
  GtkMetricType Metric;
} TRulerSettings;

// What are we doing when moving mouse: Nothing - mouse is up, dragging object,
// creating new rectangle object, selecting rectangle
typedef enum { TM_NOTHING, TM_DRAGGING, TM_DRAGHANDLE, TM_NEWRECT, TM_SELECTRECT } TMode;  

class TCADView {
protected:
  int MagA,MagB; // Magnifying constants

  GtkItemFactory *Factory,*SubFactory;
  GtkAccelGroup *AccelGroup;

  // Stores last-known visibility information
  GdkVisibilityState VisibilityState;

  GtkWidget *Window,*Fixed;
	GtkScrolledWindow *ScrolledWindow;
  GtkFrame *Frame;
  GtkVRuler *VRuler;
  GtkHRuler *HRuler;

  gboolean ResetOnUp;   // Try to reset mode on ButtonRelease
  int DownX,DownY,FocusX,FocusY;  // Position where mouse has gone down
	int SGCorrX,SGCorrY;	// Correction of selection to align
  int OldX,OldY;    // Old position - to recognize when mouse position has changed
  THandleType DHType;   // Type of handle we are dragging
  TCADHandle *Handle;	// Handle we are dragging
  gboolean ButtonDown;	// Mouse button is down
  TMode Mode;

  // For temporary storage before gdk window is created
  int WndX,WndY;

  // Updates FocusRect to contain rectangle (x,y,DownX,DownY) and draws it.
  void UpdateFocusRect(int x,int y);

  GtkCheckMenuItem *MIUseGrid,*MIShowGrid;
public:
  TCADView(TCADSheet *Sheet,char Visible);
  ~TCADView();

  GdkGC *GC;

  TCADView *Next,*Prev;
  TCADSheet *Sheet;

  GdkRectangle FocusRect;
  int XCorr,YCorr;  // X/Y correction for grid

  // Show
  void Show();
  void Hide();

  GdkDrawable *GetDrawable();
  GdkWindow *GetWindow();
  GtkWidget *GetGtkWindow();

  // Colors
  GdkColor BlueColor,BlackColor,WhiteColor;

  // Draws focus rectangle
  void DrawFocusRect(int MagA,int MagB);

  void GetMag(int& A,int& B);
  void SetMag(int A,int B);
	void ZoomIn();
	void ZoomOut();

  // Ruler settings
  TRulerSettings VRulerSettings, HRulerSettings;
  void ApplyRulerSettings();

  // Sets window titls
  void SetTitle(char *Title);

  void SetVisibilityState(GdkVisibilityState State);  // Visibility state changed
  void GotFocus();  // Window got focus
  gboolean HasFocus();  // Returns if window has focus

  virtual void ButtonPressed(int x,int y,int State,int Button);
  virtual void ButtonReleased();
  virtual void MouseMoved(int x,int y);
  virtual void HandleClicked(int x,int y,TCADHandle *Handle);

  // Sets clipping and clears area (may draw grid)
  virtual void Clear(GdkRectangle *Rect,int MagA,int MagB);

  // May draw focus rectangle
  virtual void Draw(int MagA,int MagB);

  void BringToFront();

	// Refreshes checkitems in the popup menu
	void RefreshCheckItems();

	// Shows/hides rulers
	void SetRulersVisibility(int Visibile);

	// Sets up view sizes and positions
	void SetupWindow(int WndX,int WndY,int WndW,int WndH);

	// Sets up canvas
	void SetupCanvas(int CanvasWidth,int CanvasHeight);
};

#endif

