/**********************\
*                      *
*  View menu routines  *
*                      *
\**********************/

#include <gtk/gtkitemfactory.h>
#include <gtk/gtkcheckmenuitem.h>

#include "view.h"
#include "sheet.h"
#include "main.h"
#include "rulerdialog.h"
#include "aligndialog.h"

void
VMA_UseGrid(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_UseGrid()\n");

//	printf("%d\n",View->MIUseGrid->active);
	View->Sheet->SetUseGrid(GTK_CHECK_MENU_ITEM(widget)->active);
}

void
VMA_ShowGrid(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_ShowGrid()\n");

//	printf("%d\n",View->MIShowGrid->active);
	View->Sheet->SetShowGrid(GTK_CHECK_MENU_ITEM(widget)->active);
}

void
VMA_Rulers(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;

	DTP("VMA_Rulers()\n");

	ShowRulerDialog(View);
}

void
VMA_New(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_New()\n");

  System->Menu_New();
}

void
VMA_Open(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Open()\n");

  System->Menu_Open();
}

void
VMA_Close(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Close()\n");

  System->CloseSheet(View->Sheet);
}

void
VMA_Exit(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Exit()\n");

  System->Menu_Exit();
}

void
VMA_Save(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Save()\n");

	System->Save(View->Sheet);
}

void
VMA_SaveAs(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_SaveAs()\n");

	System->SaveAs(View->Sheet);
}

void
VMA_SaveAll(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_SaveAll()\n");

	System->Menu_SaveAll();
}

void
VMA_Export_Latex(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Latex()\n");

	System->Menu_Export_Latex(View->Sheet);
}

void
VMA_Export_EPS(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_EPS()\n");

	System->Menu_Export_EPS(View->Sheet);
}

void
VMA_Export_FIG(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_FIG()\n");

	System->Menu_Export_FIG(View->Sheet);
}

void
VMA_Delete(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Delete()\n");

	View->Sheet->DeleteSelected();
}

void
VMA_SelectAll(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADView *View = (TCADView *)callback_data;
	DTP("VMA_SelectAll()\n");

	View->Sheet->SelectAll();
}

void 
VMA_Align(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Align()\n");

	ShowAlignDialog(View->Sheet);
}

void
VMA_EditText(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_EditText()\n");

	View->Sheet->EditText(false);
}

void
VMA_ArrowProperties(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_ArrowProperties()\n");

	View->Sheet->ShowArrowProperties();
}

void
VMA_TextProperties(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_TextProperties()\n");

	View->Sheet->ShowTextProperties();
}

void
VMA_LineProperties(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_LineProperties()\n");

	View->Sheet->ShowLineProperties();
}

void 
VMA_GridOptions(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_GridOptions()\n");

	View->Sheet->ShowGridProperties();
}

void 
VMA_Size(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  TCADView *View = (TCADView *)callback_data;
	DTP("VMA_Size()\n");

	View->Sheet->SheetSizeDialog();
}

void
VMA_NewView(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	TCADSheet *Sheet = ((TCADView *)callback_data)->Sheet;

	DTP("VMA_NewView()\n");

  // Create view
	Sheet->BlockChange++;
	new TCADView(Sheet,true);
	Sheet->BlockChange--;
}

void
VMA_CloseView(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_CloseView()\n");

	System->Menu_CloseWindow();
}

void
VMA_ZoomIn(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_ZoomIn()\n");

	((TCADView *)callback_data)->ZoomIn();
}

void
VMA_ZoomOut(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_ZoomOut()\n");

	((TCADView *)callback_data)->ZoomOut();
}

void
VMA_ZoomReset(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_ZoomReset()\n");

	((TCADView *)callback_data)->SetMag(1,1);
}

void
VMA_Undo(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Undo()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->Undo();
}

void
VMA_Redo(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Redo()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->Redo();
}

void
VMA_Cut(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Cut()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->EditCut();
}

void
VMA_Copy(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Copy()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->EditCopy();
}

void
VMA_Paste(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_Paste()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->EditPaste();
}

void
VMA_SendToBack(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_SendToBack()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->SendToBack();
}

void
VMA_BringToFront(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
	DTP("VMA_BringToFront()\n");

	TCADView *View = (TCADView *)callback_data;
	View->Sheet->BringToFront();
}

